// FileIO.h

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#ifndef FileIOh
#define FileIOh
/*
In C++ there seem to be many ways to open files, copy them, delete them...
Sometimes certain ways don't work and you end up wondering why - then it turns out to be a problem with long file names or something else rediculous.
But Windows itself uses a nice little dialog that gives a (random?) "time left estimate" and a progress bar.
Its can delete Directory Trees, Read only folders and files, and delete to the recycle bin....
So shouldn't we all be using this?
Heres a little class that wraps the Windows Shell magic.

To delete a File or Folder (which doesn't need to be empty):

  CFileIO::Delete("Test");

If you had a list of Directory Trees to delete, use a string of strings (double null terminator):

  CFileIO::Delete(CString("Test")+'\0'+"Test2"+'\0');

If you give the full path, CFileIO will delete to the Recycle Bin (unless you say:

  CFileIO::Delete("Test\0Test2\0",false);

The following code gets the full path for a file:

  CString Path;
  DWORD Length=GetFullPathName(Path,0,0,0); //Make relative paths absolute:
  GetFullPathName(Path, Length, Path.GetBufferSetLength(Length), 0);
  Path.ReleaseBuffer();

This also Move and Copy lists of files giving the standard Windows Move/Copy Progress Bar Dialog.
Since my implementation is meant for an Application to use, there is no user feedback with regard to Read-Only files etc.
Everything that can be Moved, Deleted or Copied will be.
For example,
  Copy("C:\\a.txt\0C:\\b.txt\0","C:\\c.txt\0C:\\d.txt\0");
copies a.txt to c.txt
and    b.txt to d.txt
The shell uses the double null terminator to identify the end of the source/destination lists.
The trailing double nulls are added by these routines, so you don't have to.
So you can copy a single file as you'd expect to:
  Copy("C:\\a.txt","C:\\b.txt");

FlushDrive Empties Cached data so you can remove Removable Media without loosing data.
*/

//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> CFO <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

class CFO { // File Object
  CString Name;
  CString ShortName;
  __int64 Size;
  DWORD   Attributes;
  CTime   Created;
  CTime   Accessed;
  CTime   Modified;

public:
  CFO() : Attributes(0) {};
  CFO(CString Directory) : Name(Directory), Attributes(FILE_ATTRIBUTE_DIRECTORY) {}
  CFO(const WIN32_FIND_DATA& FindFileData) {Set(FindFileData);}

  void Clear() {Name.Empty();}
  void Set(const WIN32_FIND_DATA& FindFileData) {
    Name      =FindFileData.cFileName;
    ShortName =FindFileData.cAlternateFileName;
    Size      =(((__int64)(FindFileData.nFileSizeHigh))<<32)|FindFileData.nFileSizeLow;
    Attributes=FindFileData.dwFileAttributes;
    Created   =FindFileData.ftCreationTime;
    Accessed  =FindFileData.ftLastAccessTime;
    Modified  =FindFileData.ftLastWriteTime;
  }

  CString GetName()       const {return Name;}
  CString GetShortName()  const {return ShortName.IsEmpty() ? Name : ShortName;}
  __int64 GetSize()       const {return Size;}
  DWORD   GetAttributes() const {return Attributes;}
  CTime   GetCreated()    const {return Created;}
  CTime   GetAccessed()   const {return Accessed;}
  CTime   GetModified()   const {return Modified;}

  bool IsReadOnly  () const {return (Attributes & FILE_ATTRIBUTE_READONLY           )!=0;}
  bool IsHidden    () const {return (Attributes & FILE_ATTRIBUTE_HIDDEN             )!=0;}
  bool IsSystem    () const {return (Attributes & FILE_ATTRIBUTE_SYSTEM             )!=0;}
  bool IsDirectory () const {return (Attributes & FILE_ATTRIBUTE_DIRECTORY          )!=0;}
  bool IsArchive   () const {return (Attributes & FILE_ATTRIBUTE_ARCHIVE            )!=0;}
  bool IsEncrypted () const {return (Attributes & FILE_ATTRIBUTE_ENCRYPTED          )!=0;}
  bool IsNormal    () const {return (Attributes & FILE_ATTRIBUTE_NORMAL             )!=0;}
  bool IsTemporary () const {return (Attributes & FILE_ATTRIBUTE_TEMPORARY          )!=0;}
  bool IsSparse    () const {return (Attributes & FILE_ATTRIBUTE_SPARSE_FILE        )!=0;}
  bool IsReparse   () const {return (Attributes & FILE_ATTRIBUTE_REPARSE_POINT      )!=0;}
  bool IsCompressed() const {return (Attributes & FILE_ATTRIBUTE_COMPRESSED         )!=0;}
  bool IsOffLine   () const {return (Attributes & FILE_ATTRIBUTE_OFFLINE            )!=0;}
  bool IsIndexed   () const {return (Attributes & FILE_ATTRIBUTE_NOT_CONTENT_INDEXED)==0;}

  static CString FormatPath(const CString S, bool TrailingBackSlash=true) { // Assumes this is a Folder. If S is a File, slashes will still be appended!
    DWORD Length=GetFullPathName(S,0,0,0); //Make relative paths absolute:
    CString Result;
    char* ptr=Result.GetBufferSetLength(Length+1); //+1 for appended back-slash
    GetFullPathName(S, Length, ptr, 0);
    if(Length>0) {
      ptr+=Length;
      *--ptr=0; //NULL Terminate the Path
      if(TrailingBackSlash) {
        if(*--ptr!='\\') { // We want a TrailingBackSlash
          *++ptr='\\';     // if we don't have one, then add one.
          *++ptr=0;
        }
      }else if(*--ptr=='\\') *ptr=0; // Watch out for this! it is useful until only a drive is specified: "C:\" means the root of drive C, and "C:" means the current working directory of drive C...
    }
    Result.ReleaseBuffer();
    return Result; // Anything using TrailingBackSlash=false should not use "C:" to get the current working Directory!
  }

  int Compare(const CFO& FO) const {      // For TreeList::Insert
    int i=FO.IsDirectory()-IsDirectory(); // Directories first.
    return i ? i : Name.CompareNoCase(FO.Name);
  }
  friend CArchive& __stdcall operator<<(CArchive& ar, const CFO& FO) {
    ar << FO.Name << FO.ShortName << FO.Attributes << FO.Created << FO.Accessed << FO.Modified << (DWORD)(FO.Size>>32) << (DWORD)(FO.Size);
    return ar;
  }
  friend CArchive& __stdcall operator>>(CArchive& ar, CFO& FO) {
    DWORD H,L;
    ar >> FO.Name >> FO.ShortName >> FO.Attributes >> FO.Created >> FO.Accessed >> FO.Modified >> H >> L;
    FO.Size=(((__int64)H)<<32)|L;
    return ar;
  }
};

//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> CFileIO <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

#include "CSV.h"

class CFileIO {
public:
  CFileIO() {}
  virtual ~CFileIO() {}

  static CString SetCurrentDirectory(CString Path) {
    CCSV CSV(0, Path, '\\'); // Change directories one-by-one:
    if(Path.Left(2)=="\\\\") { // \\ComputerName\Share\Directory format
      CSV.GetNextField(); //Skip the first two "empty fields"
      CSV.GetNextField();
      CSV.GetNextField(); //Win9x needs the ComputerName and Share to be used together, so we have to skip the ComputerName too.
    }
    while(CSV.GetNextField() && ::SetCurrentDirectory(CSV.GetDone())); // This will stop if an invalid directory is in the Path.
    return CSV.GetDone();
  }

  static bool CreatePath(CString Path) { //returns the path that was successfully created.
    CCSV CSV(0, Path, '\\');
    if(Path.Left(2)=="\\\\") { // \\ComputerName\Share\Directory format
      CSV.GetNextField(); //Skip the first two "empty fields"
      CSV.GetNextField();
      CSV.GetNextField(); //Win9x needs the ComputerName and Share to be used together, so we have to skip the ComputerName too.
    }
    while(CSV.GetNextField()
    && ((GetFileAttributes(CSV.GetDone())!=-1)
    || ::CreateDirectory(CSV.GetDone(),0)));
    return CSV.GetEnd().IsEmpty()!=0;
  }

  static CString GetCurrentDirectory() {
    CString S;
    DWORD Len=::GetCurrentDirectory(0,0);
    ::GetCurrentDirectory(Len, S.GetBufferSetLength(Len));
    S.ReleaseBuffer();
    return CFO::FormatPath(S);
  }

  static void FlushDrive(char Drive) {
    char Path[]="\\\\.\\c:";
    Path[4]=Drive;
    HANDLE hFile;
    hFile=CreateFile(Path, 0, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
    if(hFile!=INVALID_HANDLE_VALUE) FlushFileBuffers(hFile);
    CloseHandle(hFile);
  }

/*
// GetLongPathName exists only from Win98/NT5 onwards.
// We can do it with IShellFolder::ParseDisplayName instead.
// Jim Barry, Thermoteknix Systems Ltd., Cambridge, UK.
  static DWORD ShortToLongPathName(LPCTSTR ShortPath, LPTSTR LongPath, DWORD Buffer) {

    USES_CONVERSION;

    // Get the Desktop Folder
    CComPtr<IShellFolder> pDesktop;
    SHGetDesktopFolder(&pDesktop);
    if(!pDesktop) return 0;

    // Get the ID list for the short pathname
    ULONG eaten=0;
    LPITEMIDLIST item=NULL;
    pDesktop->ParseDisplayName(NULL, NULL, T2W(ShortPath), &eaten, &item, NULL);
    if(!item) {
      SetLastError(ERROR_FILE_NOT_FOUND);
      return 0;
    }

    // Get the long path of the item
    TCHAR szBuffer[_MAX_PATH]={0};
    SHGetPathFromIDList(item, szBuffer);

    // Free the pidl!
    CComPtr<IMalloc> pMalloc;
    SHGetMalloc(&pMalloc);
    if(pMalloc) pMalloc->Free(item);

    // Do what GetLongPathName does
    lstrcpyn(LongPath, szBuffer, Buffer);
    return lstrlen(szBuffer);
  }
*/

  static bool ForceDelete(CString Path, bool Recycle=true) {
    if(Delete(Path, Recycle)) return true;
    if(Path.Find('\0')==Path.GetLength()) return DeleteFile(Path)==TRUE; // Single file
    const char* S=Path;
    bool OK=true;
    while(*S) {
      if(GetFileAttributes(S)==-1) OK=false;
      else if(Delete(S, Recycle));  //Try to recycle is required
      else if(DeleteFile(S)); //Last try
      else OK=false;
      while(*S++) if((S-Path)>Path.GetLength()) return OK; // Point to next Path
    }
    return OK;
  }
  enum ActionType {aCopy,aMove,aDelete,aRecycle};
  static bool Copy(const CString& src, const CString& dst) {return File(src+'\0', dst+'\0', aCopy);}
  static bool Move(const CString& src, const CString& dst) {return File(src+'\0', dst+'\0', aMove);}
  static bool Delete(CString Path, bool Recycle=true)      {return File(Path+'\0', "", Recycle ? aRecycle : aDelete);}
private:
  static bool File(const CString& src, const CString& dst, ActionType Action) {
    if(!*src) return false;
    SHFILEOPSTRUCT FIO;
    memset(&FIO, 0, sizeof SHFILEOPSTRUCT);
    FIO.fFlags=FOF_NOCONFIRMATION|FOF_NOCONFIRMMKDIR;
    switch(Action) {
      case aRecycle: FIO.fFlags |=  FOF_ALLOWUNDO; // ..send to Recycle Bin
      case aDelete:  FIO.wFunc=FO_DELETE; break;
      case aCopy:    FIO.wFunc=FO_COPY; break;
      case aMove:    FIO.wFunc=FO_MOVE; break;
    }
    if(src.Find('\0')!=src.GetLength()) FIO.fFlags|=FOF_MULTIDESTFILES;
    FIO.pFrom=&*src;
    if(Action<aDelete) FIO.pTo=&*dst;
    return SHFileOperation(&FIO)==0;
  }
/* The old way -single file copy:
  static bool Copy(CString src, CString dst) {
    DWORD Length=GetFullPathName(src,0,0,0); //Make relative paths absolute:
    GetFullPathName(src, Length, src.GetBufferSetLength(Length), 0);
    src.ReleaseBuffer();
    Length=GetFullPathName(dst,0,0,0);
    GetFullPathName(dst, Length, dst.GetBufferSetLength(Length), 0);
    dst.ReleaseBuffer();

    HANDLE iFile=CreateFile(src, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
    if(!iFile) return false;
    HANDLE oFile=CreateFile(dst, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
    if(!oFile) {
      CloseHandle(iFile);
      return false;
    }
    DWORD iBytes,oBytes;
    char Buffer[4096];
    do {
      ReadFile (iFile, Buffer, sizeof Buffer, &iBytes, NULL);
      WriteFile(oFile, Buffer, iBytes       , &oBytes, NULL);
      if(iBytes!=oBytes) {
        CloseHandle(iFile);
        CloseHandle(oFile);
        return false;
      }
    }while(iBytes);

    BY_HANDLE_FILE_INFORMATION Info;
    GetFileInformationByHandle(iFile,&Info);
    SetFileTime(oFile,&Info.ftCreationTime,&Info.ftLastAccessTime,&Info.ftLastWriteTime);

    CloseHandle(iFile);
    CloseHandle(oFile);
    SetFileAttributes(dst,Info.dwFileAttributes);
    FlushDrive(*dst);
    return true;
  }
*/
};

#endif //ndef FileIOh
